'use strict';

exports.__esModule = true;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _lodash = require('lodash');

var _testRunTracker = require('./test-run-tracker');

var _testRunTracker2 = _interopRequireDefault(_testRunTracker);

var _builderSymbol = require('./builder-symbol');

var _builderSymbol2 = _interopRequireDefault(_builderSymbol);

var _replicator = require('./replicator');

var _observation = require('../test-run/commands/observation');

var _testRun = require('../test-run');

var _testRun2 = _interopRequireDefault(_testRun);

var _compileClientFunction = require('../compiler/es-next/compile-client-function');

var _compileClientFunction2 = _interopRequireDefault(_compileClientFunction);

var _runtime = require('../errors/runtime');

var _typeAssertions = require('../errors/runtime/type-assertions');

var _message = require('../errors/runtime/message');

var _message2 = _interopRequireDefault(_message);

var _getCallsite = require('../errors/get-callsite');

var _getCallsite2 = _interopRequireDefault(_getCallsite);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_EXECUTION_CALLSITE_NAME = '__$$clientFunction$$';

var ClientFunctionBuilder = function () {
    function ClientFunctionBuilder(fn, options) {
        var callsiteNames = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        (0, _classCallCheck3.default)(this, ClientFunctionBuilder);

        this.callsiteNames = {
            instantiation: callsiteNames.instantiation,
            execution: callsiteNames.execution || DEFAULT_EXECUTION_CALLSITE_NAME
        };

        if ((0, _lodash.isNil)(options)) options = {};

        this._validateOptions(options);

        this.fn = fn;
        this.options = options;
        this.compiledFnCode = this._getCompiledFnCode();

        if (!this.compiledFnCode) throw this._createInvalidFnTypeError();

        this.replicator = (0, _replicator.createReplicator)(this._getReplicatorTransforms());
    }

    ClientFunctionBuilder._resolveContextTestRun = function _resolveContextTestRun() {
        var testRunId = _testRunTracker2.default.getContextTestRunId();

        return _testRun2.default.activeTestRuns[testRunId];
    };

    ClientFunctionBuilder.prototype._decorateFunction = function _decorateFunction(clientFn) {
        var _this = this;

        clientFn[_builderSymbol2.default] = this;

        clientFn.with = function (options) {
            if ((typeof options === 'undefined' ? 'undefined' : (0, _typeof3.default)(options)) === 'object') options = (0, _lodash.assign)({}, _this.options, options);

            var builder = new _this.constructor(_this.fn, options, {
                instantiation: 'with',
                execution: _this.callsiteNames.execution
            });

            return builder.getFunction();
        };
    };

    ClientFunctionBuilder.prototype.getFunction = function getFunction() {
        var builder = this;

        var clientFn = function __$$clientFunction$$() {
            var testRun = builder.options.boundTestRun ? builder.options.boundTestRun.testRun : ClientFunctionBuilder._resolveContextTestRun();

            var callsite = (0, _getCallsite2.default)(builder.callsiteNames.execution);
            var args = [];

            // OPTIMIZATION: don't leak `arguments` object.
            for (var i = 0; i < arguments.length; i++) {
                args.push(arguments[i]);
            }return builder._executeCommand(args, testRun, callsite);
        };

        this._decorateFunction(clientFn);

        return clientFn;
    };

    ClientFunctionBuilder.prototype.getCommand = function getCommand(args) {
        var encodedArgs = this.replicator.encode(args);
        var encodedDependencies = this.replicator.encode(this.getFunctionDependencies());

        return this._createTestRunCommand(encodedArgs, encodedDependencies);
    };

    // Overridable methods


    ClientFunctionBuilder.prototype.getFunctionDependencies = function getFunctionDependencies() {
        return this.options.dependencies || {};
    };

    ClientFunctionBuilder.prototype._createTestRunCommand = function _createTestRunCommand(encodedArgs, encodedDependencies) {
        return new _observation.ExecuteClientFunctionCommand({
            instantiationCallsiteName: this.callsiteNames.instantiation,
            fnCode: this.compiledFnCode,
            args: encodedArgs,
            dependencies: encodedDependencies
        });
    };

    ClientFunctionBuilder.prototype._getCompiledFnCode = function _getCompiledFnCode() {
        if (typeof this.fn === 'function') return (0, _compileClientFunction2.default)(this.fn.toString(), this.options.dependencies, this.callsiteNames.instantiation, this.callsiteNames.instantiation);

        return null;
    };

    ClientFunctionBuilder.prototype._createInvalidFnTypeError = function _createInvalidFnTypeError() {
        return new _runtime.ClientFunctionAPIError(this.callsiteNames.instantiation, this.callsiteNames.instantiation, _message2.default.clientFunctionCodeIsNotAFunction, (0, _typeof3.default)(this.fn));
    };

    ClientFunctionBuilder.prototype._executeCommand = function _executeCommand(args, testRun, callsite) {
        var _this2 = this;

        if (!testRun) throw new _runtime.ClientFunctionAPIError(this.callsiteNames.execution, this.callsiteNames.instantiation, _message2.default.clientFunctionCantResolveTestRun);

        var command = this.getCommand(args);

        return testRun.executeCommand(command, callsite).then(function (result) {
            return _this2.replicator.decode(result);
        });
    };

    ClientFunctionBuilder.prototype._validateOptions = function _validateOptions(options) {
        (0, _typeAssertions.assertNonNullObject)(this.callsiteNames.instantiation, '"options" argument', options);

        if (!(0, _lodash.isNil)(options.boundTestRun)) {
            // NOTE: we can't use strict `t instanceof TestController`
            // check due to module circular reference
            if (!(options.boundTestRun.testRun instanceof _testRun2.default)) throw new _runtime.APIError(this.callsiteNames.instantiation, _message2.default.invalidClientFunctionTestRunBinding);
        }

        if (!(0, _lodash.isNil)(options.dependencies)) (0, _typeAssertions.assertObject)(this.callsiteNames.instantiation, '"dependencies" option', options.dependencies);
    };

    ClientFunctionBuilder.prototype._getReplicatorTransforms = function _getReplicatorTransforms() {
        return [new _replicator.FunctionTransform(this.callsiteNames)];
    };

    return ClientFunctionBuilder;
}();

exports.default = ClientFunctionBuilder;
module.exports = exports['default'];