'use strict';

exports.__esModule = true;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.assertNonNegativeNumber = assertNonNegativeNumber;
exports.assertBoolean = assertBoolean;
exports.assertStringOrRegExp = assertStringOrRegExp;
exports.assertObject = assertObject;
exports.assertNonNullObject = assertNonNullObject;
exports.assertFunctionOrString = assertFunctionOrString;
exports.assertFunctionOrStringOnNonNegativeNumber = assertFunctionOrStringOnNonNegativeNumber;

var _lodash = require('lodash');

var _ = require('./');

var _message = require('./message');

var _message2 = _interopRequireDefault(_message);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isNonNegativeNumber(value) {
    return (0, _lodash.isFinite)(value) && value >= 0;
}

function assertNonNegativeNumber(callsiteName, what, value) {
    if (!isNonNegativeNumber(value)) {
        var valueType = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);
        var actual = valueType === 'number' ? value : valueType;

        throw new _.APIError(callsiteName, _message2.default.valueIsNotANonNegativeNumber, what, actual);
    }
}

function assertBoolean(callsiteName, what, value) {
    var type = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);

    if (type !== 'boolean') throw new _.APIError(callsiteName, _message2.default.valueIsNotABoolean, what, type);
}

function assertStringOrRegExp(callsiteName, what, value) {
    var type = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);

    if (type !== 'string' && !(0, _lodash.isRegExp)(value)) throw new _.APIError(callsiteName, _message2.default.valueIsNotAStringOrRegExp, what, type);
}

function assertObject(callsiteName, what, value) {
    var type = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);

    if (type !== 'object') throw new _.APIError(callsiteName, _message2.default.valueIsNotAnObject, what, type);
}

function assertNonNullObject(callsiteName, what, value) {
    var type = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);

    if ((0, _lodash.isNil)(value) || type !== 'object') {
        var actualVal = value === null ? 'null' : type;

        throw new _.APIError(callsiteName, _message2.default.valueIsNotAnObject, what, actualVal);
    }
}

function assertFunctionOrString(callsiteName, what, value) {
    var type = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);

    if (type !== 'string' && type !== 'function') throw new _.APIError(callsiteName, _message2.default.valueIsNotAFunctionOrString, what, type);
}

function assertFunctionOrStringOnNonNegativeNumber(callsiteName, what, value) {
    var type = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);

    if (type !== 'string' && type !== 'function' && (type !== 'number' || !isNonNegativeNumber(value))) throw new _.APIError(callsiteName, _message2.default.valueIsNotAFunctionOrStringOrNonNegativeNumber, what, type);
}